# -*- Mode: rpm-spec -*-
# 	$Id: kernel-2.4.spec,v 1.1 2001/09/25 08:55:44 chmou Exp chmou $

# First: Get versions names right is complicated (tm)
# Second: kernel allways have the version & release number in the name
#         this is done to preserving a working kernel when you install 
#         a new one
# Stored in %rpmrversion & %rpmrelease
#
# We want to reflect in the kernel naming:
#   - vanilla kernel in with is based (ex. 2.4.21) 
#     this version is stored in %realversion
#   - Number of Mandrake kernel based on this vanilla kernel (ex. 2mdk).
#     stored in %mdkrelease.
#    
# That gives us a nice name: 2.4.21.2mdk
# 
# As version and release are allways fixed, name of the package is 
# going to be:
#        kernel-2.4.21.2mdk-1-1mdk
#
# Confused already?
#
# Now there arrive pre/rc kernels.  This kernels have the particularity
# that they have the name of the new kernel, but they are based in the
# tar file of the old kernel, i.e. names are basically:
#  - vanilla kernel: 2.4.21
#  - patch name: 2.4.22-pre3
#    stored in %use_patch
#  - Number of Mandrake kernel based in this pre kernel 2mdk
#
# That gives us a nice name again: 2.4.22.pre3.2mdk
# Problems now are:
#   - sublevel of vanilla kernel 21 (needed for tar file)
#   - sublevel of kernel is 22 (needed for prepatch and naming 
#     the package).
# This explains the need of %tar_version, because this will be different
# of %real_version if there is a pre/rc patch.
# 
# Still with me?
# 
# There are still a problem, when real 2.4.22 cames out, it will have
# a mane like: 
#              2.4.22.1mdk
# this name is (for rpm ordering of versions) smaller than:
#              2.4.22.pre3.2mdk
# to fix that we add a 0 to the name (and appear the need of realrelease)
# in the pre/rc
# 	       2.4.22.0.pre3.2mdk
#
# Take one aspirine.  Relax.
#
# Problem now is that names are just really ugly, and specially
# lilo/grub names are very difficult to read/type.
# Notice that the extra .0 is there only to make visual comparations 
# easy, but it is an annoyance.
#
# Lilo name for this kernel will be:
#       2422-0.pre3.2mdk
# And as everybody knows, putting dot's in lilo names is not nice.
# Then we change the kernel name (for loaders, and directory names only)
# to a more userfriendly:
# 		2.4.22pre3-2mdk
# That way, smp, enterprise versions continue to have the well known names
# in the loader of:
#		2422pre3-2smp
#
# This is the reason of having %kernelrelease.
# As you know, all programers are lazy.  And they don't like to have 
# the same errors once and more times.
# The following rpm/shell engineering is used to be able to calculate
# all the names exposed previosly from only the following 3 variables.
# If you don't have a pre/rc patch, define %use_patch to 0, otherwise
# define it to the patch name (i.e. pre3 or rc2).
# 
# For a non pre/rc kernel define (real examples in brakets)
#     2.4.X.Ymdk [2.4.22.2mdk]
# where
#	%sublvel = X 	[22]
#   	%mdkrelease = Y [2]
#	%use_patch 0	[0]
#
# For a pre/rc kernel do:
#	2.4.X.0.Y.Zmdk [2.4.22.0.pre3.2mdk]
# where
#	%sublvel = X 	[22]
#   	%mdkrelease = Z [2]
#	%use_patch Y	[pre3]
#
# I hope this is all clear now. If you have any doubt, please mail me at:
#
# Juan Quintela <quintela@mandrakesoft.com>
#
####
# Basically the multimedia kernel follows the same naming scheme. Except that
# I will increase the mdkrelease number sometimes, but still be based on a 
# previous 'juanrelease' for the main patch file.
#
# Danny Tholen <obiwan@mailmij.org>
###

%define sublevel 21
%define juanrelease 13
%define mdkrelease 16
%define use_patch pre4

# You shouldn't have to change any kernel/patch/version number
# for 2.4 kernels

# When we are using a pre/rc patch, the tarboll is a sublevel -1
%if %use_patch
%define tar_version 2.4.%(expr %sublevel - 1)
%define patch_version 2.4.%{sublevel}-%use_patch
%define patchversion %{use_patch}q%{juanrelease}
%define realrelease 0.%{mdkrelease}mdk
%define kernelrelease -0.%{mdkrelease}mm-mdk
%else
%define tar_version 2.4.%sublevel
%define patchversion q%mdkrelease
%define realrelease %{mdkrelease}mdk
%define kernelrelease -%{mdkrelease}mm-mdk
%endif

# never touch the folowing two fields
%define rpmversion 1
%define rpmrelease 1mdk
%define realversion 2.4.%{sublevel}
%define mdkversion %{realversion}.%{realrelease}
%define patches_ver 2.4.%{sublevel}-%{patchversion}

#The name
%define mm_name kernel-multimedia

# having different top level names for packges means
# that you have to remove them by hard :(
%define top_dir_name %{mm_name}-2.4

%define build_dir ${RPM_BUILD_DIR}/%top_dir_name

%define src_dir %{build_dir}/linux-%tar_version
%define KVERREL %{realversion}%{kernelrelease}

#dt: up is for testing only (it disables preempt+lowlat), rhconfig 
#is patched to substitute up with preempt and smp with preempt
#smp, so module building with up will have problems!!!
%define build_up 0
%define build_smp 1
%define build_preempt 1
# this is the config that contains all the drivers for the hardware/
# things that I use (Juan Quintela).
%define build_minimal 0
# If you have a broke dsdt, put a file called dsdt.hex 
# in SOURCES/ and it will compile with it.
%define build_mydsdt 0
%define build_acpi 1

%define build_kheaders 0
%define build_kgcc 0
%define build_gcc296 0
%define build_gcc30 0
%define build_gcc31 0
%define build_gcc32 0
%define build_gcc 1
%define build_debug 0
%define build_source 1

%define build_80 0
%define build_81 0
%define build_82 0
%define build_90 0
# if we don't tell aanything, we build for the last version
%define build_91 1

%ifarch alpha x86_64 ia64
%define build_enterprise 0
%endif

%ifarch x86_64
%define build_BOOT 1
%endif



# End of user definitions
%{?_without_up: %{expand: %%define build_up 0}}
%{?_without_smp: %{expand: %%define build_smp 0}}
%{?_without_preempt: %{expand: %%define build_preempt 0}}
%{?_without_minimal: %{expand: %%define build_minimal 0}}
%{?_without_mydsdt: %{expand: %%define build_mydsdt 0}}
%{?_without_kgcc: %{expand: %%define build_kgcc 0}}
%{?_without_gcc296: %{expand: %%define build_gcc296 0}}
%{?_without_gcc30: %{expand: %%define build_gcc30 0}}
%{?_without_gcc31: %{expand: %%define build_gcc31 0}}
%{?_without_gcc32: %{expand: %%define build_gcc32 0}}
%{?_without_debug: %{expand: %%define build_debug 0}}
%{?_without_source: %{expand: %%define build_source 0}}

%{?_with_up: %{expand: %%define build_up 1}}
%{?_with_smp: %{expand: %%define build_smp 1}}
%{?_with_preempt: %{expand: %%define build_preempt 1}}
%{?_with_minimal: %{expand: %%define build_minimal 1}}
%{?_with_mydsdt: %{expand: %%define build_mydsdt 1}}
%{?_with_kgcc: %{expand: %%define build_kgcc 1}}
%{?_with_gcc296: %{expand: %%define build_gcc296 1}}
%{?_with_gcc30: %{expand: %%define build_gcc30 1}}
%{?_with_gcc31: %{expand: %%define build_gcc31 1}}
%{?_with_gcc32: %{expand: %%define build_gcc32 1}}
%{?_with_debug: %{expand: %%define build_debug 1}}
%{?_with_acpi: %{expand: %%define build_acpi 1}}
%{?_with_source: %{expand: %%define build_source 1}}

%{?_with_kheaders: %{expand: %%define build_kheaders 1}}
%{?_with_80: %{expand: %%define build_80 1}}
%{?_with_81: %{expand: %%define build_81 1}}
%{?_with_82: %{expand: %%define build_82 1}}
%{?_with_90: %{expand: %%define build_90 1}}

%if %build_80
%define build_81 0
%define build_82 0
%define build_90 0
%define build_distro "8.0"
%endif

%if %build_81
%define build_80 0
%define build_82 0
%define build_90 0
%define build_distro "8.1"
%endif

%if %build_82
%define build_80 0
%define build_81 0
%define build_90 0
%define build_distro "8.2"
%endif

%if %build_90
%define build_distro "9.0"
%endif

# This language don't like too much nested elses & ifs
# Notice that gcc-2.96 needs to be last as it is defined by default.

%if %{build_kgcc}
%define KCC /usr/bin/kgcc
%define build_gcc296 0
%define build_gcc30 0
%define build_gcc31 0
%define build_gcc32 0
%define build_gcc 0
%endif

%if %{build_gcc296}
%define KCC /usr/bin/gcc
%define build_kgcc 0
%define build_gcc30 0
%define build_gcc31 0
%define build_gcc32 0
%define build_gcc 0
%endif

%if %{build_gcc30}
%define KCC /usr/bin/gcc-%(gcc3-version)
%define build_kgcc 0
%define build_gcc296 0
%define build_gcc31 0
%define build_gcc32 0
%define build_gcc 0
%endif

%if %{build_gcc31}
%define KCC /usr/bin/gcc-%(gcc3.1-version)
%define build_kgcc 0
%define build_gcc296 0
%define build_gcc30 0
%define build_gcc32 0
%define build_gcc 0
%endif

%if %{build_gcc32}
%define KCC /usr/bin/gcc-%(gcc3.2-version)
%define build_kgcc 0
%define build_gcc296 0
%define build_gcc30 0
%define build_gcc31 0
%define build_gcc 0
%endif

# default gcc should be last one

%if %{build_gcc}
%define KCC /usr/bin/gcc
%endif

%define kmake %make CC=%{KCC}
# there are places where parallel make don't work
%define smake make CC=%{KCC}

#
# kernel-multimedia: the preempt/lowlat/setpcap kernel
#
Summary: A preemptible Linux kernel, which reduces the latency of the kernel.
Name: %{mm_name}-%{mdkversion}
Version: %{rpmversion}
Release: %{rpmrelease}
License: GPL
Group: System/Kernel and hardware

#someday I should make a ppc version
ExclusiveArch: %{ix86}  
ExclusiveOS: Linux
URL: http://www.kernel.org/

####################################################################
#
# Sources
#
Source0: ftp://ftp.kernel.org/pub/linux/kernel/v2.4/linux-%{tar_version}.tar.bz2
Source4: README.kernel-sources
Source5: README.Mandrake

Source15: linux-multimedia-mdkconfig.h
Source16: linux-merge-config.awk
Source17: linux-merge-modules.awk

%if %build_mydsdt
Source50: dsdt.hex
%endif

Source100: linux-%{patches_ver}.tar.bz2

####################################################################
#
# Patches

#
# Patch0 to Patch100 are for core kernel upgrades.
#

# Pre linus patch: ftp://ftp.kernel.org/pub/linux/kernel/v2.4/testing

%if %use_patch
Patch1: patch-%patch_version.bz2
%endif

#kernel-mm patches:
Patch101: preempt-kernel-rml-2.4.21-pre1-2.patch.bz2
Patch102: 2.4.21-0.pre4.1mdk.low_latency-2.patch.bz2
#Patch103: supermount-preempt.patch.bz2
Patch104: 2.4.21-pre4q11-config-multimedia.patch.bz2
Patch105: 2.4.21-0.pre4.1mdk.cap_setpcap.patch.bz2

#extra patches:
#improve supermount (thx again andrey):
Patch111: supermount-1.1.0-2.4.21-0.11mdk.patch.bz2
Patch112: supermount-1.1.1-2.4.21-0.13mdk.patch.bz2
#fix audigy2
Patch113: 2.4.21-0.14mdk-audigy2.patch.bz2
#fix emu10k1
Patch114: 2.4.21-0.13mdk-sndemu10k1.patch.bz2
#fix dmiscan:
Patch115: 2.4.21-0.13mdk-dmiscan.patch.bz2

#END
####################################################################

# Defines for the things that are needed for all the kernels
%if %build_82
%define requires modutils >= 2.4.13-3mdk, mkinitrd >= 3.1.6-24mdk
%else
%define requires modutils >= 2.4.15-1mdk, mkinitrd >= 3.1.6-24mdk
%endif
%define prereq initscripts >= 6.40.2-44.1mdk%define requires modutils >= 2.4.3-2mdk, mkinitrd >= 3.1.6-24mdk
%define prereq initscripts >= 5.83-4mdk
%define conflicts kernel-pcmcia-cs <= 2.4.8-26mdk, iptables <= 1.2.4-1mdk
%define kprovides kernel-multimedia = %{realversion}, alsa

BuildRoot: %{_tmppath}/%{name}-%{realversion}-build
Provides: %kprovides, module-info
Autoreqprov: no
Prereq: %prereq
Requires: %requires
Conflicts: %conflicts
#Obsoletes: alsa, hackkernel, adiusbadsl_kernel
#Provides: alsa, hackkernel, adiusbadsl_kernel
BuildRequires: libbinutils2-devel, bison, perl
BuildRequires: flex zlib-devel
BuildRequires: autoconf

%description
This kernel includes patches useful for multmedia purposes like:
preemption, low-latency and the ability for processes to transfer their
capabilities.
The preemtion patches allow a task to be preempted anywhere within the kernel,
using spinlocks as markers for non-preemptibility regions. The resulting
system response is greatly increased, with measured average latencies under
1ms. Andrew Morton's low-latency patches fix the remaining points in the kernel 
that cause latency. The setpcap patch allows suid root processess to transfer 
capabilities to non-root processess, and so making it possible for user 
processes to run with realtime priority.

#
# kernel-smp: Symmetric MultiProcessing kernel
#

%package -n %{mm_name}-smp-%{mdkversion}
Summary: A multiprocessor version of a preemptive, low-latency kernel
Group: System/Kernel and hardware
Provides: %kprovides
Prereq: %prereq
Requires: %requires

%description -n %{mm_name}-smp-%{mdkversion}
This package includes a SMP version of the multimedia-kernel. It is
required only on machines with two or more CPUs, although it should work
fine on single-CPU boxes.
This kernel includes patches useful for multmedia purposes like:
preemption, low-latency and the ability for processes to transfer their
capabilities.

#
# kernel-up:  a kernel without preempt/lowlat. For testing.
#

%package -n %{mm_name}-up-%{mdkversion}
Summary: The Linux kernel (the core of the Linux operating system).
Group: System/Kernel and hardware
Provides: %kprovides
Prereq: %prereq
Requires: %requires

%description -n %{mm_name}-up-%{mdkversion}
The kernel package contains the Linux kernel (vmlinuz), the core of your
Mandrake Linux operating system.  The kernel handles the basic functions
of the operating system:  memory allocation, process allocation, device
input and output, etc.
non-root processess.
This kernel includes the setpcap patch.

%package -n %{mm_name}-source
Version: %{realversion}
Release: %{realrelease}
Requires: glibc-devel, ncurses-devel, make, gcc
Summary: The source code for the Linux kernel.
Group: Development/Kernel
Obsoletes: alsa-source
Provides: alsa-source

%description -n %{mm_name}-source
This kernel sources package is specifically for use with the multimedia
kernel, which includes preemption and low-latency patches.

The kernel-source package contains the source code files for the Linux
kernel. These source files are needed to build most C programs, since
they depend on the constants defined in the source code. The source
files can also be used to build a custom kernel that is better tuned to
your particular hardware, if you are so inclined (and you know what you're
doing).


#
# End packages - here begins build stage
#
%prep
%setup -q -n %top_dir_name -c

%setup -q -n %top_dir_name -D -T -a100

%define patches_dir ../%{patches_ver}/

cd %src_dir
%if %use_patch
%patch1 -p1
%endif


%{patches_dir}/scripts/apply_patches

#dt: I keep my patches seperate
%patch101 -p1
%patch102 -p1
#%patch103 -p1
#careful, if this patch fails, building will continue!
(cd %{patches_dir}
%patch104 -p1
)
%patch105 -p1

%patch111 -p1
%patch112 -p1
%patch113 -p1
%patch114 -p1
%patch115 -p1

# PATCH END
#
# Setup Begin
#

# Prepare all the variables for calling create configs

%if %build_debug
%define debug --debug
%else
%define debug --no-debug
%endif

%if %build_acpi
%define acpi --acpi
%else
%define acpi --no-acpi
%endif

%if %build_minimal
%define minimal --minimal
%else
%define minimal --no-minimal
%endif

%if %build_mydsdt
%define mydsdt --mydsdt
install -m 644 %{SOURCE50}  drivers/acpi
%else
%define mydsdt --no-mydsdt
%endif

%{patches_dir}/scripts/create_configs %debug %acpi %minimal %mydsdt --cpu="%{_target_cpu}"

# make sure the kernel has the sublevel we know it has...
LC_ALL=C perl -p -i -e "s/^SUBLEVEL.*/SUBLEVEL = %{sublevel}/" Makefile

# get rid of unwanted files
find . -name '*~' -o -name '*.orig' -o -name '*.append' |xargs rm -f

%if %build_kheaders

kheaders_dirs=`echo $PWD/include/{asm-*,linux,sound}`

pushd %build_dir
install -d kernel-headers/
cp -a $kheaders_dirs kernel-headers/
tar cf kernel-headers-%mdkversion.tar kernel-headers/
bzip2 -9f kernel-headers-%mdkversion.tar
rm -rf kernel-headers/
# build_kheaders
%endif


%build
# Common target directories
%define _kerneldir /usr/src/linux-%{KVERREL}
%define _bootdir /boot
%define _modulesdir /lib/modules
%define _savedheaders ../../savedheaders/

# Directories definition needed for building
%define temp_root %{build_dir}/temp-root
%define temp_source %{temp_root}%{_kerneldir}
%define temp_boot %{temp_root}%{_bootdir}
%define temp_modules %{temp_root}%{_modulesdir}

DependKernel() {
	flavour=$1
	echo "Make dep for kernel $flavour"
	%smake -s mrproper

	# We can't use only defconfig anyore because we have the autoconf patch,

	case $flavour in
	up)
		cp arch/%{_arch}/defconfig .config
		;;
	custom)
		cp arch/%{_arch}/defconfig-preempt .config
		;;
	*)
		cp arch/%{_arch}/defconfig-$flavour .config
		;;
	esac
	# make sure ETXRAVERSION says what we want it to say
	case $flavour in
	preempt)
		ext=""
		;;
	preemptsmp)
		ext="smp"
		;;
	*)
		ext=$flavour
		;;
	esac
	
	# make sure ETXRAVERSION says what we want it to say
	LC_ALL=C perl -p -i -e "s/^EXTRAVERSION.*/EXTRAVERSION = %{kernelrelease}$ext/" Makefile
	%smake oldconfig
	%smake dep
}

BuildKernel() {
	KernelVer=$1
	echo "Building kernel $KernelVer"

	%ifarch ppc
	%kmake zImage
	%endif

	%ifarch %{ix86} x86_64
	%kmake bzImage
	%endif

	%kmake modules

	# first make sure we are not loosing any .ver files to make mrporper's
	# removal of zero sized files.
	find include/linux/modules -size 0 | while read file ; do \
		echo > $file
	done

	## Start installing stuff
	install -d %{temp_boot}
	install -m 644 System.map %{temp_boot}/System.map-$KernelVer
	install -m 644 .config %{temp_boot}/config-$KernelVer

	%ifarch ppc
	cp -f vmlinux %{temp_boot}/vmlinuz-$KernelVer
	cp -f arch/ppc/boot/images/zImage.chrp-rs6k %{temp_boot}/vmlinuz-rs6k-$KernelVer

	%endif
	%ifarch %{ix86}
	cp -f arch/i386/boot/bzImage %{temp_boot}/vmlinuz-$KernelVer
	%endif
	%ifarch x86_64
	cp -f arch/x86_64/boot/bzImage %{temp_boot}/vmlinuz-$KernelVer
	%endif

	# modules
	install -d %{temp_modules}/$KernelVer
	%smake INSTALL_MOD_PATH=%{temp_root} KERNELRELEASE=$KernelVer modules_install 
}

SaveHeaders() {
	flavour=$1
%if %build_source
	HeadersRoot=%{temp_source}/savedheaders
	HeadersArch=$HeadersRoot/%{_target_cpu}/$flavour
	echo "Saving hearders for $flavour %{_target_cpu}"

	# deal with the kernel headers that are version specific
	install -d $HeadersArch
	install -m 644 include/linux/autoconf.h $HeadersArch/autoconf.h
	install -m 644 include/linux/version.h $HeadersArch/version.h
	mv include/linux/modules $HeadersArch
    	echo "%{_target_cpu} $flavour %{_savedheaders}%{_target_cpu}/$flavour/" >> $HeadersRoot/list
%endif
}

CreateKernel() {
	flavour=$1
	case $flavour in
	preempt)
		KernelVer=%{KVERREL}
		;;
	preemptsmp)
		KernelVer=%{KVERREL}smp
		;;
	*)
		KernelVer=%{KVERREL}$flavour
		;;
	esac
	DependKernel $flavour
	BuildKernel $KernelVer
	if [[ $flavour != BOOT ]];then	
		SaveHeaders $flavour
	fi
}

###
# DO it...
###

# Create a simulacro of %buildroot
rm -rf %{temp_root}
install -d %{temp_root}

#make sure we are in the directory
cd %src_dir

%if %build_smp
CreateKernel preemptsmp
%endif

%if %build_preempt
CreateKernel preempt
%endif

#up is currently for testing only,it disables preempt+lowlat 
%if %build_up
CreateKernel up
%endif

# We don't make to repeat the depend code at the install phase
%if %build_source
DependKernel custom
%endif

###
### install
###
%install
install -m 644 %{SOURCE4}  .
install -m 644 %{SOURCE5}  .

cd %src_dir
# Directories definition needed for installing
%define target_source %{buildroot}/%{_kerneldir}
%define target_boot %{buildroot}%{_bootdir}
%define target_modules %{buildroot}%{_modulesdir}

# We want to be able to test several times the install part
rm -rf %{buildroot}
cp -a %{temp_root} %{buildroot}

# Create directories infastructure
%if %build_source
install -d %{target_source} 

tar cf - . | tar xf - -C %{target_source}
ln -sf linux-%{KVERREL} %{buildroot}/usr/src/linux

# we remove all the source files that we don't ship

# first architecture files
for i in arm cris mips mips64 parisc ppc64 s390 s390x sh sparc sparc64; do
	rm -rf %{target_source}/arch/$i
	rm -rf %{target_source}/include/asm-$i
done
# ppc needs m68k headers
rm -rf %{target_source}/arch/m68k

# my patches dir, this should go in other dir
rm -rf %{target_source}/%{patches_ver}

# other misc files
rm -f %{target_source}/{.config.old,.depend,.hdepend}

# We need this to prevent someone doing a make *config without mrproper
touch %{target_source}/.need_mrproper

# We used to have a copy of DependKernel here
# Now, we make sure that the thing in the linux dir is what we want it to be

# We need to fix the patchs in .*depend files  after we fix the paths
find %{target_source} -name ".*depend" | \
while read file ; do
    mv $file $file.old
    sed -e "s|[^ ]*\(/usr/src/linux\)|\1|g" < $file.old > $file
    rm -f $file.old
done

# Try to put some smarter autoconf.h and version.h and modversions.h files in place
pushd %{target_source}/include/linux ; {
install -m 644 %{SOURCE15} rhconfig.h
rm -rf modules modversions.h autoconf.h version.h
# create modversions.h
cat > modversions.h <<EOF
#ifndef _LINUX_MODVERSIONS_H
#define _LINUX_MODVERSIONS_H
#include <linux/rhconfig.h>
#include <linux/modsetver.h>
EOF
list=`find %{_savedheaders}*/*/modules/*.ver -exec basename '{}' \; | sort -u`
mkdir modules
for l in $list; do
    sed 's,$,modules/'$l, %{_savedheaders}list | awk -f %{SOURCE17} > modules/$l
    touch -r modules/$l modules/`basename $l .ver`.stamp
    echo '#include <linux/modules/'$l'>' >> modversions.h
done
echo '#endif /* _LINUX_MODVERSIONS_H */' >> modversions.h
# Create autoconf.h file
echo '#include <linux/rhconfig.h>' > autoconf.h
sed 's,$,autoconf.h,' %{_savedheaders}list | awk -f %{SOURCE16} >> autoconf.h
# Create version.h
echo "#include <linux/rhconfig.h>" >> version.h
loop_cnt=0
#would be better to get rid of preempt here....:
for i in preemptsmp preempt up ; do
	if [ -d %{_savedheaders}%{_target_cpu}/$i -a \
	     -f %{_savedheaders}%{_target_cpu}/$i/version.h ]; then
		if [ $loop_cnt = 0 ]; then
      			buf="#if defined(__module__$i)"
      			previous_i="$i"
      			loop_cnt=1
    		else
      			echo "$buf" >> version.h
      			grep UTS_RELEASE %{_savedheaders}%{_target_cpu}/${previous_i}/version.h >> version.h
 	     		buf="#elif defined(__module__$i)"
      			previous_i="$i"
      			loop_cnt=`expr $loop_cnt + 1`
    		fi
    fi
done
#write last lines
if [ $loop_cnt -eq 0 ]; then
	echo "You need to build at least one kernel"
	exit 1;
fi

if [ $loop_cnt -gt 1 ]; then
	echo "#else" >> version.h
fi

grep UTS_RELEASE %{_savedheaders}%{_target_cpu}/${previous_i}/version.h >> version.h

if [ $loop_cnt -gt 1 ]; then
	echo "#endif" >> version.h
fi

# Any of the version.h are ok, as they only differ in the first line
ls %{_savedheaders}%{_target_cpu}/*/version.h | head -1 | xargs grep -v UTS_RELEASE >> version.h
rm -rf %{_savedheaders}
} ; popd
#endif build_source
%endif
# Gzip module and relink the link to a .gz module
find %{target_modules} -type f -name '*.o'|xargs gzip -9f
for i in $(find %{target_modules} -type l -name '*.o');do
    link=$(LC_ALL=C perl -e 'print readlink shift, "\n"' $i)
    ln -s $link.gz $i.gz
    rm -f $i
done

for i in %{target_modules}/*; do
  rm -f $i/build $i/modules.*
done

# sniff, if we gzipped all the modules, we change the stamp :(
# we really need the depmod -ae here

%define depmod_opts -u -ae

%if %build_80
%define depmod_opts -ae
%endif 

%if %build_81
%define depmod_opts -ae
%endif

pushd %{target_modules}
for i in *; do
	/sbin/depmod %depmod_opts -b %{buildroot} -r -F %{target_boot}/System.map-$i $i
	echo $?
done

for i in *; do
	pushd $i
	echo "Creating module.description for $i"
	modules=`find . -name "*.o" -o -name "*.o.gz"`
	/sbin/modinfo -f '%{filename} %{description}\n' $modules \
	| perl -lne 'print "$1\t$3" if m|([^/]*)\.o(\.gz)? "(.*)"|'  > modules.description
	popd
done

popd

export EXCLUDE_FROM_STRIP="vmlinuz-rs6k-%{KVERREL} vmlinuz-rs6k-%{KVERREL}smp vmlinuz-rs6k-%{KVERREL}enterprise vmlinuz-rs6k-%{KVERREL}BOOT"

###
### clean
###

%clean
rm -rf %{buildroot}
# We don't want to remove this, the whole reason of its existence is to be 
# able to do several rpm --short-circuit -bi for testing install 
# phase without repeating compilation phase
#rm -rf %{temp_root} 

###
### scripts
###

# do this for upgrades...in case the old modules get removed we have
# loopback in the kernel so that mkinitrd will work.
%preun
/sbin/modprobe loop 2> /dev/null > /dev/null

REMOVE=
if [ -f /etc/sysconfig/system ];then
		source /etc/sysconfig/system
		[[ "$CLASS" = "beginner" ]] && REMOVE=yes
fi
if [ -f /etc/sysconfig/installkernel ];then
		source /etc/sysconfig/installkernel
		[[ "$AUTOREMOVE" = [Yy][Ee][Ss] ]] && REMOVE=yes
		[[ "$AUTOREMOVE" = [Nn][Oo] ]] && REMOVE=
fi

BUILD=/lib/modules/%{KVERREL}/build
if [ -L $BUILD ]; then
       rm -f $BUILD
fi

[[ -n $REMOVE ]] && /sbin/installkernel -a -R -c %{KVERREL}

exit 0

%preun -n %{mm_name}-smp-%{mdkversion}
/sbin/modprobe loop 2> /dev/null > /dev/null

REMOVE=
if [ -f /etc/sysconfig/system ];then
		source /etc/sysconfig/system
		[[ "$CLASS" = "beginner" ]] && REMOVE=yes
fi
if [ -f /etc/sysconfig/installkernel ];then
		source /etc/sysconfig/installkernel
		[[ "$AUTOREMOVE" = [Yy][Ee][Ss] ]] && REMOVE=yes
		[[ "$AUTOREMOVE" = [Nn][Oo] ]] && REMOVE=
fi

BUILD=/lib/modules/%{KVERREL}smp/build
if [ -L $BUILD ]; then
       rm -f $BUILD
fi

[[ -n $REMOVE ]] && /sbin/installkernel -a -R -c %{KVERREL}smp

exit 0

%preun -n %{mm_name}-up-%{mdkversion}
/sbin/modprobe loop 2> /dev/null > /dev/null

REMOVE=
if [ -f /etc/sysconfig/system ];then
                source /etc/sysconfig/system
                [[ "$CLASS" = "beginner" ]] && REMOVE=yes
fi
if [ -f /etc/sysconfig/installkernel ];then
                source /etc/sysconfig/installkernel
                [[ "$AUTOREMOVE" = [Yy][Ee][Ss] ]] && REMOVE=yes
                [[ "$AUTOREMOVE" = [Nn][Oo] ]] && REMOVE=
fi

BUILD=/lib/modules/%{KVERREL}up/build
if [ -L $BUILD ]; then
       rm -f $BUILD
fi

[[ -n $REMOVE ]] && /sbin/installkernel -a -R -c %{KVERREL}up

exit 0


%post
cd %{_bootdir}
# do not link (-L)
if [ -x /sbin/installkernel ];then
	      /sbin/installkernel -a -L -c %{KVERREL}
fi

KERNEL_SOURCE=/usr/src/linux-%{KVERREL}
if [ -d $KERNEL_SOURCE ]; then
       ln -sf $KERNEL_SOURCE /lib/modules/%{KVERREL}/build
fi

%postun
cd %{_bootdir}
if [ -f initrd-%{KVERREL}.img ];then
		rm -f initrd-%{KVERREL}.img
fi

%post -n %{mm_name}-up-%{mdkversion}
cd %{_bootdir}
#do not link (-L) 
if [ -x /sbin/installkernel ];then
              /sbin/installkernel -a -L -c %{KVERREL}up
fi
KERNEL_SOURCE=/usr/src/linux-%{KVERREL}
if [ -d $KERNEL_SOURCE ]; then
       ln -sf $KERNEL_SOURCE /lib/modules/%{KVERREL}up/build
fi


%postun -n %{mm_name}-up-%{mdkversion}
cd %{_bootdir}
if [ -f initrd-%{KVERREL}up.img ];then
               rm -f initrd-%{KVERREL}up.img
fi

%post -n %{mm_name}-smp-%{mdkversion}
cd %{_bootdir}
if [ -x /sbin/installkernel ];then
	      /sbin/installkernel -a -L -c %{KVERREL}smp
fi

KERNEL_SOURCE=/usr/src/linux-%{KVERREL}
if [ -d $KERNEL_SOURCE ]; then
       ln -sf $KERNEL_SOURCE /lib/modules/%{KVERREL}smp/build
fi


%postun -n %{mm_name}-smp-%{mdkversion}
cd %{_bootdir}
if [ -f initrd-%{KVERREL}smp.img ];then
		rm -f initrd-%{KVERREL}smp.img
fi

%post -n %{mm_name}-source
cd /usr/src
rm -f linux
ln -snf linux-%{KVERREL} linux
/sbin/service kheader start 2>/dev/null >/dev/null || :
# we need to create /build only when there is a source tree.
for i in "" smp ; do
	if [ -d /lib/modules/%{KVERREL}$i ]; then
		ln -sf /usr/src/linux-%{KVERREL} /lib/modules/%{KVERREL}$i/build
	fi
done

%postun -n %{mm_name}-source
if [ -L /usr/src/linux ]; then 
    if [ -L /usr/src/linux -a `ls -l /usr/src/linux 2>/dev/null| awk '{ print $11 }'` = "linux-%{KVERREL}" ]; then
	[ $1 = 0 ] && rm -f /usr/src/linux
    fi
fi
# we need to delete <modules>/build at unsinstall
for i in "" smp ; do
	if [ -L /lib/modules/%{KVERREL}$i/build ]; then
		rm -f /lib/modules/%{KVERREL}$i/build
	fi
done
exit 0

###
### file lists
###

%if %build_up
%files -n kernel-up-%{mdkversion}
%defattr(-,root,root)
%{_bootdir}/config-%{KVERREL}up
%{_bootdir}/vmlinuz-%{KVERREL}up
%ifarch ppc
%{_bootdir}/vmlinuz-rs6k-%{KVERREL}up
%endif
%{_bootdir}/System.map-%{KVERREL}up
%dir %{_modulesdir}/%{KVERREL}up/
%{_modulesdir}/%{KVERREL}up/kernel
%{_modulesdir}/%{KVERREL}up/modules.*
%doc README.kernel-sources
%doc README.Mandrake
%endif

%if %build_smp
%files -n %{mm_name}-smp-%{mdkversion}
%defattr(-,root,root)
%{_bootdir}/config-%{KVERREL}smp
%{_bootdir}/vmlinuz-%{KVERREL}smp   
%ifarch ppc
%{_bootdir}/vmlinuz-rs6k-%{KVERREL}smp
%endif
%{_bootdir}/System.map-%{KVERREL}smp
%dir %{_modulesdir}/%{KVERREL}smp/
%{_modulesdir}/%{KVERREL}smp/kernel
%{_modulesdir}/%{KVERREL}smp/modules.*
%doc README.kernel-sources
%doc README.Mandrake
%endif

%if %build_preempt
%files -n %{mm_name}-%{mdkversion}
%defattr(-,root,root)
%{_bootdir}/config-%{KVERREL}
%{_bootdir}/vmlinuz-%{KVERREL}
%{_bootdir}/System.map-%{KVERREL}
%dir %{_modulesdir}/%{KVERREL}/
%{_modulesdir}/%{KVERREL}/kernel
%{_modulesdir}/%{KVERREL}/modules.*
%doc README.kernel-sources
%doc README.Mandrake
%endif

%if %build_source
%files -n %{mm_name}-source
%defattr(-,root,root)
%dir %{_kerneldir}
%dir %{_kerneldir}/arch
%dir %{_kerneldir}/include
%{_kerneldir}/.config
%{_kerneldir}/.need_mrproper
%{_kerneldir}/COPYING
%{_kerneldir}/CREDITS
%{_kerneldir}/Documentation
%{_kerneldir}/MAINTAINERS
%{_kerneldir}/Makefile
%{_kerneldir}/README
%{_kerneldir}/REPORTING-BUGS
%{_kerneldir}/Rules.make
%{_kerneldir}/arch/alpha
%{_kerneldir}/arch/i386
%{_kerneldir}/arch/ia64
%{_kerneldir}/arch/ppc
%{_kerneldir}/arch/x86_64
%{_kerneldir}/drivers
%{_kerneldir}/fs
%{_kerneldir}/init
%{_kerneldir}/ipc
%{_kerneldir}/kdb
%{_kerneldir}/kernel
%{_kerneldir}/lib
%{_kerneldir}/mm
%{_kerneldir}/net
%{_kerneldir}/scripts
%{_kerneldir}/sound
%{_kerneldir}/3rdparty
%{_kerneldir}/grsecurity
%{_kerneldir}/include/asm-alpha
%{_kerneldir}/include/asm-generic
%{_kerneldir}/include/asm-i386
%{_kerneldir}/include/asm-ia64
# That is needed by ppc
%{_kerneldir}/include/asm-m68k
%{_kerneldir}/include/asm-ppc
%{_kerneldir}/include/asm-x86_64
%{_kerneldir}/include/asm
%{_kerneldir}/include/linux
%{_kerneldir}/include/math-emu
%if %build_acpi
%{_kerneldir}/include/acpi
%endif
%{_kerneldir}/include/net
%{_kerneldir}/include/pcmcia
%{_kerneldir}/include/scsi
%{_kerneldir}/include/sound
%{_kerneldir}/include/video
%doc README.kernel-sources
%doc README.Mandrake
#endif %build_source
%endif

#%files -n kernel-doc
#%defattr(-,root,root)
#%doc linux-%{tar_version}/Documentation/*

%changelog
* Thu Mar 13 2003 Danny Tholen <obiwan@mailmij.org> 2.4.21.0.16mdk
- supermount 1.1.1 (a great job, by Andrey Borzenkov)
- small snd-emu10k1 patch
- dmi_scan fixes (Thomas Backlund)
- custom config should now have patches enabled

* Fri Mar 8 2003 Danny Tholen <obiwan@mailmij.org> 2.4.21.0.15mdk
- update audigy code
- sync with main kernel 2.4.21.0.13mdk

* Thu Mar 6 2003 Danny Tholen <obiwan@mailmij.org> 2.4.21.0.14mdk
- I screwed up. preempt/lowlat/setpcap should now be active again.
- sync with main kernel 2.4.21.0.12mdk

* Tue Mar 04 2003 Danny Tholen <obiwan@mailmij.org> 2.4.21.0.13mdk
- drop preempt tag, it only gave problems, we are now called mm-mdk!
- change descriptions.
- small change to source15 (__module_preempt)
- fix ntfs+drakfont issue

* Sun Mar 02 2003 Danny Tholen <obiwan@mailmij.org> 2.4.21.0.12mdk
- spec and patch fixes for preemptsmp
- supermount patch from andrey
- add provides
- rediff config patch

* Fri Feb 28 2003 Danny Tholen <obiwan@mailmij.org> 2.4.21.0.11mdk
- merged with latest kernel in main
- build smp version

* Mon Feb 17 2003 Danny Tholen <obiwan@mailmij.org> 2.4.21-0.pre4.6mdk
- Changed all names to kernel-multimedia (better name welcome)
- remove things from spec I do not see usefull in future
- remove old kernel changelog

* Mon Feb 17 2003 Danny Tholen <obiwan@mailmij.org> 1-1mdk
- Add lowlatency patch (modified from chmouels one)
- Add preempt patch
- Capabilities patch
- updated configs patch

